// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "../Asset.hpp"
#include "../AssetLoader.hpp"
#include "util/flag.hpp"
#include "display/font/truetype/NewTrueTypeFaceTexture.hpp"

namespace Dawn {
  enum NewTrueTypeAssetState {
    NEW_TRUE_TYPE_ASSET_STATE_INITIAL,
    NEW_TRUE_TYPE_ASSET_STATE_OPEN,
    NEW_TRUE_TYPE_ASSET_STATE_VALIDATE_HEADER,
    NEW_TRUE_TYPE_ASSET_STATE_VALIDATE_VERSION,
    NEW_TRUE_TYPE_ASSET_STATE_READ_VARIANT_COUNT,
    NEW_TRUE_TYPE_ASSET_STATE_READ_VARIANT,
    NEW_TRUE_TYPE_ASSET_STATE_ADJUSTING_OFFSETS,
    NEW_TRUE_TYPE_ASSET_STATE_INIT_FREETYPE,
    NEW_TRUE_TYPE_ASSET_STATE_READY
  };

  struct NewTrueTypeAssetStyle {
    flag_t style;
    size_t dataSize;
    size_t dataOffset;
  };

  class NewTrueTypeAsset : public Asset {
    protected:
      AssetLoader loader;
      FT_Library fontLibrary;
      enum NewTrueTypeAssetState state = NEW_TRUE_TYPE_ASSET_STATE_INITIAL;
      std::vector<struct NewTrueTypeAssetStyle> assetStyles;

      std::vector<NewTrueTypeFaceTexture*> textures;
      std::map<usagelockid_t, NewTrueTypeFaceTexture*> textureByLock;
      std::map<struct NewTrueTypeFaceTextureStyle, NewTrueTypeFaceTexture*> textureByStyle;

    public:
      NewTrueTypeAsset(AssetManager *assMan, std::string name);
      void updateSync() override;
      void updateAsync() override;

      usagelockid_t lock(struct NewTrueTypeFaceTextureStyle style);
      NewTrueTypeFaceTexture * getTexture(usagelockid_t lock);
      void unlock(usagelockid_t lock);

      ~NewTrueTypeAsset();
  };
}