// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "UILabelNew.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

UILabelNew::UILabelNew(SceneItem *item) :
  UIComponentRenderable(item),
  font(nullptr),
  fontSize(16),
  style(0)
{

}

void UILabelNew::rebufferQuads() {
  std::cout << "Rebuffering" << std::endl;
  this->mesh.createBuffers(QUAD_VERTICE_COUNT * 128, QUAD_INDICE_COUNT * 128);

  glm::vec2 position(32, 32);

  struct FontShaderBufferData fontData;
  auto x = this->bufferQuads(
    "Hello World",
    fontData,
    this->font->getTexture(this->fontLock),
    position,
    0,
    0
  );

  fontData.colors[0] = COLOR_MAGENTA;
  fontData.textures[0] = 0;
  // fontData.colors[1] = COLOR_RED;
  // fontData.textures[1] = 1;
  // fontData.colors[2] = COLOR_GREEN;
  // fontData.textures[2] = 2;
  shaderBuffer.buffer(&fontData);
}

void UILabelNew::onStart() {
  this->shaderBuffer.init();

  useEffect([&]{
    usagelockid_t newLock = -1;

    if(font != nullptr) {
      newLock = font->lock(NewTrueTypeFaceTextureStyle{
        fontSize,
        style
      });
    }

    if(fontLock != -1) {
      font.previous->unlock(fontLock);
      fontLock = -1;
    }

    if(font == nullptr) return;

    fontLock = newLock;
    this->rebufferQuads();
  }, font)();

  useEffect([&]{
    if(font == nullptr) return;
    auto newLock = font->lock(NewTrueTypeFaceTextureStyle{
      fontSize,
      style
    });

    if(fontLock != -1) {
      font->unlock(fontLock);
      fontLock = -1;
    }

    fontLock = newLock;
    this->rebufferQuads();
  }, { &fontSize, &style })();
}

std::vector<struct ShaderPassItem> UILabelNew::getUIRenderPasses() {
  if(this->fontLock == -1) return {};

  auto canvas = this->getCanvas();
  auto shader = getGame()->renderManager.fontShader;

  struct ShaderPassItem item;
  item.shader = shader;
  item.mesh = &this->mesh;
  item.matrixValues[shader->paramModel] = transform->getWorldTransform();
  item.parameterBuffers[shader->bufferUiCanvas] = &canvas->shaderBuffer;
  item.parameterBuffers[shader->bufferFont] = &this->shaderBuffer;
  item.renderFlags = RENDER_MANAGER_RENDER_FLAG_BLEND;

  item.textureSlots[0] = &this->font->getTexture(this->fontLock)->texture;
  item.textureValues[shader->paramTexture0] = 0;
  // item.textureSlots[1] = &defFaceItalics.texture;
  // item.textureSlots[2] = &defFaceBold.texture;
  // item.textureValues[shader->paramTexture1] = 1;
  // item.textureValues[shader->paramTexture2] = 2;

  return { item };
}


float_t UILabelNew::getWidth() {
  return 0;
}

float_t UILabelNew::getHeight() {
  return 0;
}

float_t UILabelNew::getContentWidth() {
  return 0;
}

float_t UILabelNew::getContentHeight() {
  return 0;
}

int32_t UILabelNew::bufferQuads(
  std::string text,
  struct FontShaderBufferData &bufferData,
  struct NewTrueTypeFaceTexture *texture,
  glm::vec2 &position,
  int32_t quadStart,
  int32_t partIndex
) {
  // Get string length
  int32_t len = text.length();
  
  glm::vec2 wh = glm::vec2(texture->texture.getWidth(), texture->texture.getHeight());

  // For each char
  for(int32_t i = 0; i < len; i++) {
    char ch = text[i];
    int32_t j = quadStart + i;
    FT_ULong c = ch;
    auto charInfo = texture->getCharacterData(c);

    // Determine texture coordinates.
    glm::vec2 uv0 = glm::vec2(0.0f, charInfo.textureY) / wh;
    glm::vec2 uv1 = uv0 + (charInfo.bitmapSize / wh);

    // Buffer the quad.
    QuadMesh::bufferQuadMeshWithZ(&this->mesh,
      position + charInfo.bitmapPosition, uv0,
      position + charInfo.bitmapPosition + charInfo.bitmapSize, uv1,
      0.0f,
      j * QUAD_VERTICE_COUNT, j * QUAD_INDICE_COUNT
    );

    // Move the current position along.
    position.x += charInfo.advanceX;
    position.y += charInfo.advanceY;

    // Set the part index to the quad mappings
    bufferData.fontQuadMappings[j] = partIndex;
  }

  return len;
}